
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003, 2011, 2015 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gEKGdisplay.h"

// ----------------------------------------------------------------------
//
//  Konstruktor und Destruktor
// 
// ----------------------------------------------------------------------

/* Konstruktor */
gEKGdisplay::gEKGdisplay() : Gtk::DrawingArea(), Values(MinWidth - GapWidth), 
                             Plast(MinWidth-1),
			     ZLcolor( Glib::ustring( "red" ) ),
			     BGcolor( Glib::ustring( "white" ) ),
			     FGcolor( Glib::ustring( "black" ) )
{ 
  mVmax = ( EKG_amp[0] + EKG_amp[1] + EKG_amp[2] ) / 3 * 255;
  // Gre Initalisieren
  set_size_request( MinWidth, MinWidth ); 
  for ( int i = 0; i < 3; i++ )
    zeroPos[i] = MinWidth / 12 + i * MinWidth / 6;
  // Farben Reservieren
  Glib::RefPtr< Gdk::Colormap > cmap = Gdk::Colormap::get_system();
  cmap->alloc_color( BGcolor );
  cmap->alloc_color( FGcolor );
  cmap->alloc_color( ZLcolor );
  // Pipe zur Prozekommunikation
  if ( 0 != pipe( Pipe ) ) 
    throw string("gEKGdisplay::gEKGdisplay() : Can't open pipe.");
  Glib::signal_io().connect( sigc::mem_fun(*this, &gEKGdisplay::drawRealtime), 
                             Pipe[0], Glib::IO_IN );
}


/* Destruktor */
gEKGdisplay::~gEKGdisplay() { }


// ----------------------------------------------------------------------
//
// berschriebene Methoden von Widget
// 
// ----------------------------------------------------------------------

/* Erstbenutzung und gre ndern. */
bool gEKGdisplay::on_configure_event(GdkEventConfigure* p0)
{
  win = get_window();
  win->set_background( BGcolor );
  Values.resize( get_width() - GapWidth );
  for ( int i = 0; i < 6; i++ )
    zeroPos[i] = get_height() / 12 + i * get_height() / 6;
  if ( Plast >= get_width() ) 
    Plast = get_width() - 1;
  return true; // Event erfolgreich behandelt
}


/* berschriebene Funktion von Widget. Zeichnet das Display. */
bool gEKGdisplay::on_expose_event(GdkEventExpose* p0)
{
  Glib::RefPtr<Gdk::GC> gc = gc->create( win );
  gc->set_background( BGcolor );
  win->clear_area_e( p0->area.x, p0->area.y, p0->area.width, p0->area.height );
  gc->set_clip_rectangle( Gdk::Rectangle(&p0->area) );

  gint x1 = addX( Plast, GapWidth + 1 );
  gint x2;
  gint y[12];
  gint * yswap;
  gint * y1 = y; 
  gint * y2 = y + 6;
  scaleY( Values[0], y1 );
  for ( int i = 1; i < Values.size(); i++)
  {
    scaleY( Values[i], y2 );
    x2 = addX( x1, 1 );
    if ( x2 != 0 ) // nicht ber den rechten Rand malen
      for ( int k = 0; k < 6; k++ )
      {
        gc->set_foreground( ZLcolor );
        win->draw_line( gc, x1, zeroPos[k], x2, zeroPos[k] );
        gc->set_foreground( FGcolor );
        win->draw_line( gc, x1, y1[k], x2, y2[k] );
      }
    x1 = x2;
    yswap = y1;
    y1 = y2;
    y2 = yswap;
  }
  // TODO gc->set_noclip();
  return true; // Event erfolgreich behandelt
}


// ----------------------------------------------------------------------
//
// berschriebene Methoden EKGdevice
// 
// ----------------------------------------------------------------------

/* Daten annehmen, speichern und darstellen */
void gEKGdisplay::input( float * d )
{
  if ( dsize != write( Pipe[1], d, dsize ) )
    throw string("gEKGdisplay::input() : Can't write dsize bytes to pipe.");
  EKGdevice::input( d );
}


// ----------------------------------------------------------------------
//
// Public Methoden
// 
// ----------------------------------------------------------------------

/* Alle Werte Lschen */
void gEKGdisplay::reset()
{
  Values.reset();
  Plast = get_width();
  //draw( Gdk::Rectangle( 0, 0, get_width(), get_height() ) );
  win->clear();
}

// ----------------------------------------------------------------------
//
// Private Methoden
// 
// ----------------------------------------------------------------------


// ----------------------------------------------------------------------
// Darstellung
// ----------------------------------------------------------------------

/* Echtzeit Darstellung. Wird von Gtk::Main::input event aufgerufen. 
 * Die Daten werden aus der Pipe gelesen. */
bool gEKGdisplay::drawRealtime( Glib::IOCondition )
{
  // Daten von Pipe holen
  float d[3];
  if ( dsize != read( Pipe[0], d, dsize ) )
    throw string("gEKGdisplay::drawRealtime() : Can't get dsize bytes.");
  // Zeichnen
  gint y1[6];
  gint y2[6];
  gint x2 = addX( Plast, 1 );
  Glib::RefPtr<Gdk::GC> gc = gc->create( win );
  // zeichnen, es sei denn, wir sind am rechten Rand.
  if ( Plast != get_width() - 1 )
  {
    gc->set_clip_rectangle( Gdk::Rectangle( Plast, 0, 2, get_height() ) );
    scaleY( Values[ Values.size()-1 ], y1 );
    scaleY( d, y2 );
    for ( int i = 0; i < 6; i++ )
    {
      gc->set_foreground( ZLcolor );
      win->draw_line( gc, Plast, zeroPos[i], x2, zeroPos[i] );
      gc->set_foreground( FGcolor );
      win->draw_line( gc, Plast, y1[i], x2, y2[i] );
    }
  }
  // lteste Werte vom Bildschirm lschen.
  gc->set_foreground( BGcolor );
  int oldVal = addX( Plast, GapWidth );
  gc->set_clip_rectangle( Gdk::Rectangle( oldVal, 0, 1, get_height() ) );
  win->draw_line( gc, oldVal, 0, oldVal, get_height() );
  // TODO gc->set_noclip();
  // Bildschirmposition aktualisieren, Werte speichern.
  Plast = x2;
  Values.input( d );
  return true;
}


// ----------------------------------------------------------------------
// Skalierung
// ----------------------------------------------------------------------

// Alle sechs y-Postitonen
void gEKGdisplay::scaleY( float * y, gint * Y )
{
  Y[0] = scaleY( y[0], 0 );
  Y[1] = scaleY( y[1], 1 );
  Y[2] = scaleY( y[2], 2 );
  Y[3] = scaleY( aVR( y ), 3 );
  Y[4] = scaleY( aVL( y ), 4 );
  Y[5] = scaleY( aVF( y ), 5 );
}

